Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.OutputModeForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dSUID === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSUID"));
            return;
        }

        this.dSUID = config.dSUID;
        this.dialogType = config.type;
        this.group = config.group;
        this.matrix = config.matrix;
        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.radio = undefined;

        var items = [];

        this.magicNumber_rb1 = 661;
        this.magicNumber_rb2 = 662;
        if (this.matrix.heatingoutmode === true) {
            items =
            [
                {
                    boxLabel: dSS._("switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: this.magicNumber_rb1.toString(),
                    inputValue: this.magicNumber_rb1
                },
                {
                    boxLabel: dSS._("pulse width modulation (PWM)"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: this.magicNumber_rb2.toString(),
                    inputValue: this.magicNumber_rb2
                },
                {
                    boxLabel: dSS._("disabled"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                    inputValue: dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                }
            ];
        } else if (this.matrix.extradimmer === true) {
            items =
            [
                {
                    boxLabel: dSS._("switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_SWITCH.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_SWITCH
                },
                {
                    boxLabel: dSS._("dimmed 0-10V"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DIMMER_0_10V.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DIMMER_0_10V
                },
                {
                    boxLabel: dSS._("dimmed 1-10V"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DIMMER_1_10V.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DIMMER_1_10V
                },
                {
                    boxLabel: dSS._("disabled"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                }
            ];
        } else {
            items =
            [
                {
                    boxLabel: dSS._("switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_SWITCH.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_SWITCH
                },
                {
                    boxLabel: dSS._("dimmed"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DIMMER.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DIMMER
                },
                {
                    boxLabel: dSS._("disabled"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                }
            ];
        }

        this.radio = new Ext.form.RadioGroup(
        {
            bubbleEvents: [ 'change' ],
            fieldLabel: dSS._("Output mode"),
            columns: 1,
            items: items
        });


        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            scope: this,
            items: this.radio
        });

        dSS.devprops.OutputModeForm.superclass.constructor.apply(this, arguments);

        if (this.matrix.heatingoutmode === true) {
            dSS.MessageBus.addListener('heatinggroupformchanged',
                this.updateGroup.createDelegate(this), this);
        }

        this.on('loadcomplete', function() {
                this.on('change', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.radio.markInvalid(data.json.message);
                    this.fireEvent('formloaded', true);
                    return;
                }

                var val = data.json.result[this.dSUID][0].outputMode;
                this.setRadioGroup(val);

                this.el.unmask();
                this.fireEvent('formloaded', true);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
                if (this.matrix.heatingoutmode === true) {
                    dSS.MessageBus.removeListener('heatinggroupformchanged',
                        this.updateGroup.createDelegate(this), this);
                }
            },
            this);

        this.on('submit', function() {
            var value;
            // this happens if the value was not loaded correctly
            if (this.radio.getValue() != null) {
                value = this.radio.getValue().getRawValue();
                value = this.convertValueFromMagic(value);
            }

            // do not submit anything if value was not loaded and also has
            // not been changed by the user
            if ((value === undefined) || (value == this.loadedValue)) {
                this.fireEvent('formsubmitted');
                return;
            }

            var dSUID = this.dSUID;

            // this should never happen, but lets double check so we
            // do not attempt to write some random crap into the klemme
            var validValue = true;
            if (this.matrix.heatingoutmode === true) {
                if ((value !== dSS.config.deviceParams.OUTPUT_MODE_PWM) &&
                    (value !== dSS.config.deviceParams.OUTPUT_MODE_CONTROL_PWM) &&
                    (value !== dSS.config.deviceParams.OUTPUT_MODE_SWITCH) &&
                    (value !== dSS.config.deviceParams.OUTPUT_MODE_DISABLED) &&
                    (value !== dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED)) {

                    validValue = false;
                }
            } else if (this.matrix.extradimmer === true) {
                if ((value !== dSS.config.deviceParams.OUTPUT_MODE_SWITCH) &&
                    (value !== dSS.config.deviceParams.OUTPUT_MODE_DIMMER_0_10V) &&
                    (value !== dSS.config.deviceParams.OUTPUT_MODE_DIMMER_1_10V) &&
                    (value !== dSS.config.deviceParams.OUTPUT_MODE_DISABLED)) {

                    validValue = false;
                }
            } else {
                if ((value !== dSS.config.deviceParams.OUTPUT_MODE_SWITCH) &&
                    (value !== dSS.config.deviceParams.OUTPUT_MODE_DISABLED) &&
                    (value !== dSS.config.deviceParams.OUTPUT_MODE_DIMMER)) {
                    validValue = false;
                }
            }

            if (!validValue) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Attempted to set invalid value") +
                              ": " + value);
                this.fireEvent('formsubmitted');
                return;
            }

            dSS.util.genericRequest({
                    url: '/json/device/setOutputMode',
                    params:
                    {
                        dsuid : this.dSUID,
                        modeID : value
                    },
                    failureMsg: dSS._("Could not save output mode settings"),
                    scope: this,
                    evt: 'formsubmitted'
                },
                function(success) {
                    if (success === true) {
                        var devStore = dSS.data.Loader.getDeviceStore();
                        var record = devStore.getById(dSUID);
                        if (record !== undefined) {
                            record.set('outputMode', parseInt(value, 10));
                            record.commit();
                        }
                    }
                }
            );
        },
        this);
    },
    updateGroup: function(group) {
        if (this.matrix.heatingoutmode !== true) {
            return;
        }

        this.group = group;
    },
    setRadioGroup: function(value) {
        if (this.matrix.heatingoutmode === true) {
            if ((value === dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED) ||
                (value === dSS.config.deviceParams.OUTPUT_MODE_SWITCH)) {
                value = this.magicNumber_rb1;
            } else if (dSS.util.isHeatingPWMMode(value)) {
                value = this.magicNumber_rb2;
            }
        }

        this.radio.setValue(value, value);
    },

    convertValueFromMagic: function(value) {
        value = parseInt(value);

        if (this.matrix.heatingoutmode !== true) {
            return value;
        }

        if (value === this.magicNumber_rb1) {
            if (this.group === dSS.config.groups.TEMPCONTROL) {
                return dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED;
            } else {
                return dSS.config.deviceParams.OUTPUT_MODE_SWITCH;
            }
        } else if (value === this.magicNumber_rb2) {
            if (this.group === dSS.config.groups.TEMPCONTROL) {
                return dSS.config.deviceParams.OUTPUT_MODE_CONTROL_PWM;
            } else {
                return dSS.config.deviceParams.OUTPUT_MODE_PWM;
            }
        }

        return value; // 0 = disabled is same for both groups
    }
});

Ext.reg('dssdevoutputmodeform', dSS.devprops.OutputModeForm);

